<?php

namespace Drupal\entity_reference_pagination_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceEntityFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Plugin implementation of the 'entity reference rendered entity' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_pagination_formatter",
 *   label = @Translation("Rendered entity (Paginated)"),
 *   description = @Translation("Display the referenced entities rendered and paginated)."),
 *   field_types = {
 *     "entity_reference",
 *   }
 * )
 */
class EntityReferencePaginationFormatter extends EntityReferenceEntityFormatter {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $currentRequest;

  /**
   * Constructs an EntityReferenceEntityFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin ID for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, LoggerChannelFactoryInterface $logger_factory, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository, RequestStack $request_stack) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $logger_factory, $entity_type_manager, $entity_display_repository);

    $this->currentRequest = $request_stack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('logger.factory'),
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $settings = parent::defaultSettings();
    $settings += [
      'index_name' => 'page',
      'items_per_page' => 10,
      'ajaxlink_method' => 'replace',
      'ajaxlink_auto' => FALSE,
      'ajaxlink_history' => FALSE,
      'ajaxlink_remove_after_execution' => FALSE,
    ];
    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements['index_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Index field name'),
      '#default_value' => $this->getSetting('index_name'),
      '#required' => TRUE,
    ];
    $elements['items_per_page'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Number of items per page'),
      '#default_value' => $this->getSetting('items_per_page'),
      '#required' => TRUE,
    ];
    $elements['ajaxlink_method'] = [
      '#type' => 'select',
      '#title' => $this->t('Method to display results'),
      '#default_value' => $this->getSetting('ajaxlink_method'),
      '#options' => [
        'replace' => $this->t('Replace'),
        'append' => $this->t('Append'),
      ],
      '#required' => TRUE,
    ];
    $elements['ajaxlink_auto'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Auto load next page when link is displayed.'),
      '#default_value' => $this->getSetting('ajaxlink_auto'),
    ];
    $elements['ajaxlink_history'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Add pagination events to history'),
      '#default_value' => $this->getSetting('ajaxlink_history'),
    ];
    $elements['ajaxlink_remove_after_execution'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Remove link after execution'),
      '#default_value' => $this->getSetting('ajaxlink_remove_after_execution'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $summary[] = $this->t('Pagination index: @index_name', ['@index_name' => $this->getSetting('index_name')]);
    $summary[] = $this->t('Items per page: @items_per_page', ['@items_per_page' => $this->getSetting('items_per_page')]);
    $ajaxlink_method = $this->getSetting('ajaxlink_method');
    $method = 'replaced';
    switch ($ajaxlink_method) {
      case 'append':
        $method = 'appended';
        break;
    }
    $summary[] = $this->t('Result is @method to existing content', ['@method' => $method]);
    if ($this->getSetting('ajaxlink_auto')) {
      $summary[] = $this->t('Link is automatically triggered');
    }
    if ($this->getSetting('ajaxlink_history')) {
      $summary[] = $this->t('Follow ajax link history');
    }
    if ($this->getSetting('ajaxlink_remove_after_execution')) {
      $summary[] = $this->t('Link is removed after ajax execution');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    $items_per_page = $this->getSetting('items_per_page');
    $index_name = $this->getSetting('index_name');

    $index_current = (int) $this->currentRequest->query->get($index_name, 0);

    $elements = [
      '#cache' => [
        'contexts' => ['url.path', 'url.query_args'],
      ],
    ];

    // Get the entity IDs of the items to be able to filter them.
    $items_entity_ids = [];
    foreach ($items as $index => $item) {
      $items_entity_ids[$index] = $item->entity->id();
    }
    // Chunk items entity IDs to get the items for the current page.
    $items_entity_ids_chunked = array_chunk($items_entity_ids, $items_per_page);
    if (!isset($items_entity_ids_chunked[$index_current])) {
      return $elements;
    }
    $items_entity_ids_current_page = $items_entity_ids_chunked[$index_current];

    // Filter the items to only get the ones for the current page.
    $items_current_page = $items->filter(function ($item) use ($items_entity_ids_current_page) {
      return in_array($item->entity->id(), $items_entity_ids_current_page);
    });
    $elements = parent::view($items_current_page, $langcode);

    $wrapperId = "field--name-{$this->fieldDefinition->getName()}--ajax-wrapper--{$this->getSetting('index_name')}";
    // If chunk from the next index exists, add a link to the next page.
    if (isset($items_entity_ids_chunked[$index_current + 1])) {
      $url = Url::fromRoute('<current>', [], [
        'query' => [$index_name => $index_current + 1] + $this->currentRequest->query->all(),
      ]);

      $nbItems = $elements['#items']->count();
      // Be sure to not override the existing elements.
      while (isset($elements[$nbItems])) {
        $nbItems++;
      }

      $elements[$nbItems] = [
        '#type' => 'link',
        '#title' => $this->t('Next page'),
        '#url' => $url,
        '#attributes' => [
          'class' => ['ajax-link'],
          // The selector should be the wrapper of the field with random ID.
          'data-ajax-link-selector' => "#$wrapperId",
          'data-ajax-link-method' => $this->getSetting('ajaxlink_method'),
        ],
        '#attached' => [
          'library' => ['ajax_link/ajaxLink'],
        ],
      ];
      if ($this->getSetting('ajaxlink_auto')) {
        $elements[$nbItems]['#attributes']['class'][] = 'ajax-link-auto';
      }
      if ($this->getSetting('ajaxlink_history')) {
        $elements[$nbItems]['#attributes']['data-ajax-link-history'] = 1;
      }
      if ($this->getSetting('ajaxlink_remove_after_execution')) {
        $elements[$nbItems]['#attributes']['data-ajax-link-remove-after-execution'] = 1;
      }
    }

    $elements['#attributes']['id'][] = $wrapperId;
    $elements['#cache']['contexts'][] = 'url.path';
    $elements['#cache']['contexts'][] = 'url.query_args';

    return $elements;

  }

}
